"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.escapeQuery = exports.runInBatches = exports.copyTable = exports.wrapMigration = exports.loadSurveyFromDisk = void 0;
const fs_1 = require("fs");
const n8n_core_1 = require("n8n-core");
const config_1 = __importDefault(require("../../config"));
const Logger_1 = require("../../Logger");
const constants_1 = require("../../constants");
const logger = (0, Logger_1.getLogger)();
const PERSONALIZATION_SURVEY_FILENAME = 'personalizationSurvey.json';
function loadSurveyFromDisk() {
    const userSettingsPath = n8n_core_1.UserSettings.getUserN8nFolderPath();
    try {
        const filename = `${userSettingsPath}/${PERSONALIZATION_SURVEY_FILENAME}`;
        const surveyFile = (0, fs_1.readFileSync)(filename, 'utf-8');
        (0, fs_1.rmSync)(filename);
        const personalizationSurvey = JSON.parse(surveyFile);
        const kvPairs = Object.entries(personalizationSurvey);
        if (!kvPairs.length) {
            throw new Error('personalizationSurvey is empty');
        }
        else {
            const emptyKeys = kvPairs.reduce((acc, [_key, value]) => {
                if (!value || (Array.isArray(value) && !value.length)) {
                    return acc + 1;
                }
                return acc;
            }, 0);
            if (emptyKeys === kvPairs.length) {
                throw new Error('incomplete personalizationSurvey');
            }
        }
        return surveyFile;
    }
    catch (error) {
        return null;
    }
}
exports.loadSurveyFromDisk = loadSurveyFromDisk;
let runningMigrations = false;
function logMigrationStart(migrationName) {
    if (constants_1.inTest)
        return;
    if (!runningMigrations) {
        logger.warn('Migrations in progress, please do NOT stop the process.');
        runningMigrations = true;
    }
    logger.debug(`Starting migration ${migrationName}`);
}
function logMigrationEnd(migrationName) {
    if (constants_1.inTest)
        return;
    logger.debug(`Finished migration ${migrationName}`);
}
const runDisablingForeignKeys = async (migration, context, fn) => {
    const { dbType, queryRunner } = context;
    if (dbType !== 'sqlite')
        throw new Error('Disabling transactions only available in sqlite');
    await queryRunner.query('PRAGMA foreign_keys=OFF');
    await queryRunner.startTransaction();
    try {
        await fn.call(migration, context);
        await queryRunner.commitTransaction();
    }
    catch (e) {
        try {
            await queryRunner.rollbackTransaction();
        }
        catch { }
        throw e;
    }
    finally {
        await queryRunner.query('PRAGMA foreign_keys=ON');
    }
};
const wrapMigration = (migration) => {
    const dbType = config_1.default.getEnv('database.type');
    const dbName = config_1.default.getEnv(`database.${dbType === 'mariadb' ? 'mysqldb' : dbType}.database`);
    const tablePrefix = config_1.default.getEnv('database.tablePrefix');
    const migrationName = migration.name;
    const context = {
        tablePrefix,
        dbType,
        dbName,
        migrationName,
        logger,
    };
    const { up, down } = migration.prototype;
    Object.assign(migration.prototype, {
        async up(queryRunner) {
            logMigrationStart(migrationName);
            if (this.transaction === false) {
                await runDisablingForeignKeys(this, { queryRunner, ...context }, up);
            }
            else {
                await up.call(this, { queryRunner, ...context });
            }
            logMigrationEnd(migrationName);
        },
        async down(queryRunner) {
            if (down) {
                if (this.transaction === false) {
                    await runDisablingForeignKeys(this, { queryRunner, ...context }, up);
                }
                else {
                    await down.call(this, { queryRunner, ...context });
                }
            }
        },
    });
};
exports.wrapMigration = wrapMigration;
const copyTable = async ({ tablePrefix, queryRunner }, fromTable, toTable, fromFields = [], toFields = [], batchSize = 10) => {
    const driver = queryRunner.connection.driver;
    fromTable = driver.escape(`${tablePrefix}${fromTable}`);
    toTable = driver.escape(`${tablePrefix}${toTable}`);
    const fromFieldsStr = fromFields.length
        ? fromFields.map((f) => driver.escape(f)).join(', ')
        : '*';
    const toFieldsStr = toFields.length
        ? `(${toFields.map((f) => driver.escape(f)).join(', ')})`
        : '';
    const total = await queryRunner
        .query(`SELECT COUNT(*) as count from ${fromTable}`)
        .then((rows) => rows[0].count);
    let migrated = 0;
    while (migrated < total) {
        await queryRunner.query(`INSERT INTO ${toTable} ${toFieldsStr} SELECT ${fromFieldsStr} FROM ${fromTable} LIMIT ${migrated}, ${batchSize}`);
        migrated += batchSize;
    }
};
exports.copyTable = copyTable;
function batchQuery(query, limit, offset = 0) {
    return `
			${query}
			LIMIT ${limit}
			OFFSET ${offset}
		`;
}
async function runInBatches(queryRunner, query, operation, limit = 100) {
    let offset = 0;
    let batchedQuery;
    let batchedQueryResults;
    if (query.trim().endsWith(';'))
        query = query.trim().slice(0, -1);
    do {
        batchedQuery = batchQuery(query, limit, offset);
        batchedQueryResults = (await queryRunner.query(batchedQuery));
        await operation([...batchedQueryResults]);
        offset += limit;
    } while (batchedQueryResults.length === limit);
}
exports.runInBatches = runInBatches;
const escapeQuery = (queryRunner, query, params) => queryRunner.connection.driver.escapeQueryWithParameters(query, {
    pinData: params.pinData,
    id: params.id,
}, {});
exports.escapeQuery = escapeQuery;
//# sourceMappingURL=migrationHelpers.js.map