"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceControlGitService = void 0;
const typedi_1 = require("typedi");
const child_process_1 = require("child_process");
const n8n_workflow_1 = require("n8n-workflow");
const path_1 = __importDefault(require("path"));
const simple_git_1 = require("simple-git");
const constants_1 = require("./constants");
const sourceControlHelper_ee_1 = require("./sourceControlHelper.ee");
let SourceControlGitService = exports.SourceControlGitService = class SourceControlGitService {
    constructor() {
        this.git = null;
        this.gitOptions = {};
    }
    preInitCheck() {
        n8n_workflow_1.LoggerProxy.debug('GitService.preCheck');
        try {
            const gitResult = (0, child_process_1.execSync)('git --version', {
                stdio: ['pipe', 'pipe', 'pipe'],
            });
            n8n_workflow_1.LoggerProxy.debug(`Git binary found: ${gitResult.toString()}`);
        }
        catch (error) {
            throw new Error(`Git binary not found: ${error.message}`);
        }
        try {
            const sshResult = (0, child_process_1.execSync)('ssh -V', {
                stdio: ['pipe', 'pipe', 'pipe'],
            });
            n8n_workflow_1.LoggerProxy.debug(`SSH binary found: ${sshResult.toString()}`);
        }
        catch (error) {
            throw new Error(`SSH binary not found: ${error.message}`);
        }
        return true;
    }
    async initService(options) {
        const { sourceControlPreferences: sourceControlPreferences, gitFolder, sshKeyName, sshFolder, } = options;
        n8n_workflow_1.LoggerProxy.debug('GitService.init');
        if (this.git !== null) {
            return;
        }
        this.preInitCheck();
        n8n_workflow_1.LoggerProxy.debug('Git pre-check passed');
        (0, sourceControlHelper_ee_1.sourceControlFoldersExistCheck)([gitFolder, sshFolder]);
        const sshKnownHosts = path_1.default.join(sshFolder, 'known_hosts');
        const sshCommand = `ssh -o UserKnownHostsFile=${sshKnownHosts} -o StrictHostKeyChecking=no -i ${sshKeyName}`;
        this.gitOptions = {
            baseDir: gitFolder,
            binary: 'git',
            maxConcurrentProcesses: 6,
            trimmed: false,
        };
        this.git = (0, simple_git_1.simpleGit)(this.gitOptions)
            .env('GIT_SSH_COMMAND', sshCommand)
            .env('GIT_TERMINAL_PROMPT', '0');
        if (!(await this.checkRepositorySetup())) {
            await this.git.init();
        }
        if (!(await this.hasRemote(sourceControlPreferences.repositoryUrl))) {
            if (sourceControlPreferences.connected && sourceControlPreferences.repositoryUrl) {
                await this.initRepository(sourceControlPreferences);
            }
        }
    }
    resetService() {
        this.git = null;
    }
    async checkRepositorySetup() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        if (!(await this.git.checkIsRepo())) {
            return false;
        }
        try {
            await this.git.status();
            return true;
        }
        catch (error) {
            return false;
        }
    }
    async hasRemote(remote) {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        try {
            const remotes = await this.git.getRemotes(true);
            const foundRemote = remotes.find((e) => e.name === constants_1.SOURCE_CONTROL_ORIGIN && e.refs.push === remote);
            if (foundRemote) {
                n8n_workflow_1.LoggerProxy.debug(`Git remote found: ${foundRemote.name}: ${foundRemote.refs.push}`);
                return true;
            }
        }
        catch (error) {
            throw new Error(`Git is not initialized ${error.message}`);
        }
        n8n_workflow_1.LoggerProxy.debug(`Git remote not found: ${remote}`);
        return false;
    }
    async initRepository(sourceControlPreferences) {
        var _a;
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        if (sourceControlPreferences.initRepo) {
            try {
                await this.git.init();
            }
            catch (error) {
                n8n_workflow_1.LoggerProxy.debug(`Git init: ${error.message}`);
            }
        }
        try {
            await this.git.addRemote(constants_1.SOURCE_CONTROL_ORIGIN, sourceControlPreferences.repositoryUrl);
        }
        catch (error) {
            if (error.message.includes('remote origin already exists')) {
                n8n_workflow_1.LoggerProxy.debug(`Git remote already exists: ${error.message}`);
            }
            else {
                throw error;
            }
        }
        await this.git.addConfig('user.email', sourceControlPreferences.authorEmail);
        await this.git.addConfig('user.name', sourceControlPreferences.authorName);
        if (sourceControlPreferences.initRepo) {
            try {
                const branches = await this.getBranches();
                if (((_a = branches.branches) === null || _a === void 0 ? void 0 : _a.length) === 0) {
                    await this.git.raw(['branch', '-M', sourceControlPreferences.branchName]);
                }
            }
            catch (error) {
                n8n_workflow_1.LoggerProxy.debug(`Git init: ${error.message}`);
            }
        }
    }
    async getBranches() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        try {
            const { branches } = await this.git.branch(['-r']);
            const remoteBranches = Object.keys(branches)
                .map((name) => name.split('/')[1])
                .filter((name) => name !== 'HEAD');
            const { current } = await this.git.branch();
            return {
                branches: remoteBranches,
                currentBranch: current,
            };
        }
        catch (error) {
            throw new Error(`Could not get remote branches from repository ${error.message}`);
        }
    }
    async setBranch(branch) {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        await this.git.checkout(branch);
        await this.git.branch([`--set-upstream-to=${constants_1.SOURCE_CONTROL_ORIGIN}/${branch}`, branch]);
        return this.getBranches();
    }
    async fetch() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        return this.git.fetch();
    }
    async getCurrentBranch() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        const currentBranch = (await this.git.branch()).current;
        return {
            current: currentBranch,
            remote: 'origin/' + currentBranch,
        };
    }
    async diff(options) {
        var _a, _b;
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        const currentBranch = await this.getCurrentBranch();
        const target = (_a = options === null || options === void 0 ? void 0 : options.target) !== null && _a !== void 0 ? _a : currentBranch.remote;
        const dots = (_b = options === null || options === void 0 ? void 0 : options.dots) !== null && _b !== void 0 ? _b : '...';
        return this.git.diffSummary([dots + target]);
    }
    async diffRemote() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        const currentBranch = await this.getCurrentBranch();
        if (currentBranch.remote) {
            const target = currentBranch.remote;
            return this.git.diffSummary(['...' + target]);
        }
        return;
    }
    async diffLocal() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        const currentBranch = await this.getCurrentBranch();
        if (currentBranch.remote) {
            const target = currentBranch.current;
            return this.git.diffSummary([target]);
        }
        return;
    }
    async pull(options = { ffOnly: true }) {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        if (options.ffOnly) {
            return this.git.pull(undefined, undefined, { '--ff-only': null });
        }
        return this.git.pull();
    }
    async push(options = {
        force: false,
        branch: constants_1.SOURCE_CONTROL_DEFAULT_BRANCH,
    }) {
        const { force, branch } = options;
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        if (force) {
            return this.git.push(constants_1.SOURCE_CONTROL_ORIGIN, branch, ['-f']);
        }
        return this.git.push(constants_1.SOURCE_CONTROL_ORIGIN, branch);
    }
    async stage(files, deletedFiles) {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        if (deletedFiles === null || deletedFiles === void 0 ? void 0 : deletedFiles.size) {
            try {
                await this.git.rm(Array.from(deletedFiles));
            }
            catch (error) {
                n8n_workflow_1.LoggerProxy.debug(`Git rm: ${error.message}`);
            }
        }
        return this.git.add(Array.from(files));
    }
    async resetBranch(options = { hard: false, target: 'HEAD' }) {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        if (options === null || options === void 0 ? void 0 : options.hard) {
            return this.git.raw(['reset', '--hard', options.target]);
        }
        return this.git.raw(['reset', options.target]);
    }
    async commit(message) {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        return this.git.commit(message);
    }
    async status() {
        if (!this.git) {
            throw new Error('Git is not initialized');
        }
        const statusResult = await this.git.status();
        return statusResult;
    }
};
exports.SourceControlGitService = SourceControlGitService = __decorate([
    (0, typedi_1.Service)()
], SourceControlGitService);
//# sourceMappingURL=sourceControlGit.service.ee.js.map